/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "data_type.hh"
#include <stdexcept>

using namespace sends;
using namespace std;

//======================================  Conversion tables
static const char* dtype_strings[] = {
    "byte_1",
    "int_2", 
    "int_4", 
    "int_8", 
    "real_4", 
    "real_8", 
    "complex_8", 
    "complex_16", 
    0
};

//======================================  Get a data type code.
void
data_type::set(const std::string& str) {
    for (int i=0; dtype_strings[i]; i++) {
	if (str == dtype_strings[i]) {
	    mDataType = data_enum(i);
	    return;
	}
    }
    throw runtime_error(string("Unidentified data type name: ")+str);
}

//======================================  Set a data type from a frame type.
void
data_type::set_frame_type(int ftype) {
    switch (ftype) {
    case 0:  // FR_VECT_C
    case 12: // FR_VECT_1U
	mDataType = kByte1;
	break;
    case 1:  // FR_VECT_2S
    case 9:  // FR_VECT_2U
	mDataType = kInt2;
	break;
    case 2:  // FR_VECT_8R
	mDataType = kReal8;
	break;
    case 3:  // FR_VECT_4R
	mDataType = kReal4;
	break;
    case 4:  // FR_VECT_4S
    case 10: // FR_VECT_4U
	mDataType = kInt4;
	break;
    case 5:  // FR_VECT_8S
    case 11: // FR_VECT_8U
	mDataType = kInt8;
	break;
    case 6:  // FR_VECT_8C
	mDataType = kComplex8;
	break;
    case 7:  // FR_VECT_16C
	mDataType = kComplex16;
	break;
    case 8: // FR_VECT_STRING
    default:
	throw runtime_error("Unidentified frame data type");
    }
}

//======================================  Get a data type string.
int
data_type::size(void) const {
    switch (mDataType) {
    case kByte1:
	return 1;
    case kInt2:
	return 2;
    case kInt4:
	return 4;
    case kInt8:
	return 8;
    case kReal4:
	return 4;
    case kReal8:
	return 8;
    case kComplex8:
	return 8;
    case kComplex16:
	return 16;
    }
    return kInt4;
}

//======================================  Get data element size.
const char*
data_type::str(void) const {
    return dtype_strings[int(mDataType)];
}
